"""
Module: 'ure' on micropython-v1.23.0-esp32-ESP32_GENERIC_S3
"""

# MCU: {'version': '1.23.0', 'mpy': 'v6.3', 'port': 'esp32', 'board': 'ESP32_GENERIC_S3', 'family': 'micropython', 'build': '', 'arch': 'xtensawin', 'ver': '1.23.0', 'cpu': 'ESP32S3'}
# Stubber: v1.20.0
from __future__ import annotations
from _typeshed import Incomplete

def sub(*args, **kwargs) -> Incomplete: ...
def search(*args, **kwargs) -> Incomplete: ...
def match(*args, **kwargs) -> Incomplete: ...
def compile(*args, **kwargs) -> Incomplete: ...
